@@ -2,7 +2,7 @@
 //
 //    DIGIKAMAPP.CPP
 //
-//    Copyright (C) 2002-2004 Renchi Raju <renchi at pooh.tam.uiuc.edu>
+//    Copyright (C) 2002-2005 Renchi Raju <renchi at pooh.tam.uiuc.edu>
 //                            Gilles Caulier <caulier dot gilles at free.fr>
 //
 //    This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,7 @@
 #include <qstringlist.h>
 #include <qkeysequence.h>
 #include <qsignalmapper.h>
+#include <qtimer.h>
 
 // KDE includes.
 
@@ -70,14 +71,23 @@
 #include "imagepluginloader.h"
 #include "imagewindow.h"
 #include "digikamapp.h"
+#include "splashscreen.h"
 
-DigikamApp::DigikamApp() : KMainWindow( 0, "Digikam" )
+DigikamApp::DigikamApp(bool detectCamera)
+          : KMainWindow( 0, "Digikam" )
 {
     m_instance = this;
     m_config = kapp->config();
 
     mFullScreen = false;
     mView = 0;
+	
+    mSplash = 0;
+    if(m_config->readBoolEntry("Show Splash", true) &&
+       !kapp->isRestored())
+    {
+        mSplash = new SplashScreen();
+    }
 
     mAlbumSettings = new AlbumSettings();
     mAlbumSettings->readSettings();
@@ -96,12 +106,13 @@ DigikamApp::DigikamApp() : KMainWindow( 0, "Digikam" )
     setupActions();
     updateDeleteTrashMenu();
 
-    setAutoSaveSettings();
     applyMainWindowSettings(m_config);
 
     mAlbumManager->setLibraryPath(mAlbumSettings->getAlbumLibraryPath());
 
     // Load Cameras
+    if(mSplash)
+        mSplash->message(i18n("Loading cameras"), AlignLeft, white);
     loadCameras();
 
     // Load KIPI Plugins.
@@ -109,10 +120,23 @@ DigikamApp::DigikamApp() : KMainWindow( 0, "Digikam" )
 
     // Load Themes
     populateThemes();
+
+    setAutoSaveSettings();    
+    
+    // Auto-detect camera if requested so
+    if (detectCamera)
+    {
+        if(mSplash)
+            mSplash->message(i18n("Auto-detect camera"), AlignLeft, white);
+        QTimer::singleShot(0, this, SLOT(slotCameraAutoDetect()));
+    }
 }
 
 DigikamApp::~DigikamApp()
 {
+    if (ImageWindow::imagewindow())
+        delete ImageWindow::imagewindow();
+    
     if (mView)
         delete mView;
 
@@ -129,6 +153,17 @@ DigikamApp* DigikamApp::getinstance()
     return m_instance;
 }
 
+void DigikamApp::show()
+{
+    if(mSplash)
+    {
+        mSplash->finish(this);
+        delete mSplash;
+        mSplash = 0;
+    }
+    KMainWindow::show();
+}
+
 const QPtrList<KAction>& DigikamApp::menuImageActions()
 {
     return m_kipiImageActions;
@@ -184,6 +219,40 @@ void DigikamApp::setupActions()
 
     // -----------------------------------------------------------------
 
+    mBackwardActionMenu = new  KToolBarPopupAction(i18n("&Back"),
+                                    "back",
+                                    ALT+Key_Left,
+                                    mView,
+                                    SLOT(slotAlbumHistoryBack()),
+                                    actionCollection(),
+                                    "album_back");
+    mBackwardActionMenu->setEnabled(false);
+    connect(mBackwardActionMenu->popupMenu(), 
+            SIGNAL(aboutToShow()),
+            this,
+            SLOT(slotAboutToShowBackwardMenu()));
+    connect(mBackwardActionMenu->popupMenu(), 
+            SIGNAL(activated(int)),
+            mView,
+            SLOT(slotAlbumHistoryBack(int)));
+       
+    mForwardActionMenu = new  KToolBarPopupAction(i18n("Forward"),
+                                    "forward",
+                                    ALT+Key_Right,
+                                    mView,
+                                    SLOT(slotAlbumHistoryForward()),
+                                    actionCollection(),
+                                    "album_forward");
+    mForwardActionMenu->setEnabled(false);
+    connect(mForwardActionMenu->popupMenu(), 
+            SIGNAL(aboutToShow()),
+            this,
+            SLOT(slotAboutToShowForwardMenu()));
+    connect(mForwardActionMenu->popupMenu(), 
+            SIGNAL(activated(int)),
+            mView,
+            SLOT(slotAlbumHistoryForward(int)));    
+        
     mNewAction = new KAction(i18n("&New Album..."),
                                     "albumfoldernew",
                                     KStdAccel::shortcut(KStdAccel::New),
@@ -191,7 +260,8 @@ void DigikamApp::setupActions()
                                     SLOT(slot_newAlbum()),
                                     actionCollection(),
                                     "album_new");
-
+    mNewAction->setWhatsThis(i18n("This option create a new empty Album in the database."));
+                                         
     mAlbumSortAction = new KSelectAction(i18n("&Sort Albums"),
                                     0,
                                     0,
@@ -223,6 +293,7 @@ void DigikamApp::setupActions()
                                     SLOT(slot_albumAddImages()),
                                     actionCollection(),
                                     "album_addImages");
+    mAddImagesAction->setWhatsThis(i18n("This option adding new images in the current Album."));
 
     mAlbumImportAction = new KAction( i18n("Import Folders..."),
                                     "fileopen",
@@ -240,7 +311,9 @@ void DigikamApp::setupActions()
                                     SLOT(slot_albumPropsEdit()),
                                     actionCollection(),
                                     "album_propsEdit");
-
+    mPropsEditAction->setWhatsThis(i18n("This option allows you to set the Album Properties information "
+                                        "about the Collection."));
+    
     mOpenInKonquiAction = new KAction( i18n("Open in Konqueror"),
                                     "konqueror",
                                     0,
@@ -272,7 +345,9 @@ void DigikamApp::setupActions()
                                     SLOT(slot_imageView()),
                                     actionCollection(),
                                     "image_view");
-
+    mImageViewAction->setWhatsThis(i18n("This option allows you to open the Image Editor with the current selected "
+                                        "image."));
+                                        
     mImageCommentsAction = new KAction(i18n("Edit Image Comments && Tags..."),
                                    "imagecomment",
                                     Key_F3,
@@ -280,15 +355,9 @@ void DigikamApp::setupActions()
                                     SLOT(slot_imageCommentsEdit()),
                                     actionCollection(),
                                     "image_comments");
-
-    mImageViewExifAction = new KAction(i18n("View Exif Information"),
-                                    "exifinfo",
-                                    Key_F6,
-                                    mView,
-                                    SLOT(slot_imageExifInfo()),
-                                    actionCollection(),
-                                    "image_view_exif");
-
+    mImageCommentsAction->setWhatsThis(i18n("This option allows you to edit the comments and tags of the current "
+                                            "selected image."));
+                                        
     mImageRenameAction = new KAction(i18n("Rename..."),
                                     "pencil",
                                     Key_F2,
@@ -296,7 +365,9 @@ void DigikamApp::setupActions()
                                     SLOT(slot_imageRename()),
                                     actionCollection(),
                                     "image_rename");
-
+    mImageRenameAction->setWhatsThis(i18n("This option allows you to rename the filename of the current selected "
+                                          "image."));                                    
+                                    
     mImageDeleteAction = new KAction(i18n("Delete"),
                                     "editdelete",
                                     SHIFT+Key_Delete,
@@ -305,13 +376,15 @@ void DigikamApp::setupActions()
                                     actionCollection(),
                                     "image_delete");
 
-    mImagePropsAction = new KAction(i18n("Properties"),
-                                    "image",
+    mImagePropsAction = new KAction(i18n("Properties..."),
+                                    "exifinfo",
                                     ALT+Key_Return,
                                     mView,
                                     SLOT(slotImageProperties()),
                                     actionCollection(),
                                     "image_properties");
+    mImagePropsAction->setWhatsThis(i18n("This option allows you to display the file properties, the meta-data "
+                                         "and the histogram of the current selected image."));
 
     mImageSortAction = new KSelectAction(i18n("&Sort Images"),
                                     0,
@@ -419,6 +492,7 @@ void DigikamApp::setupActions()
                                    SLOT(slot_thumbSizePlus()),
                                    actionCollection(),
                                    "album_thumbSizeIncrease");
+    mThumbSizePlusAction->setWhatsThis(i18n("This option allows you to increase the Album thumbnails size."));    
 
     mThumbSizeMinusAction = new KAction(i18n("Decrease Thumbnail Size"),
                                    "viewmag-",
@@ -427,6 +501,7 @@ void DigikamApp::setupActions()
                                    SLOT(slot_thumbSizeMinus()),
                                    actionCollection(),
                                    "album_thumbSizeDecrease");
+    mThumbSizeMinusAction->setWhatsThis(i18n("This option allows you to decrease the Album thumbnails size."));    
 
     mFullScreenAction = new KAction(i18n("Toggle Full Screen"),
                                    "window_fullscreen",
@@ -435,13 +510,14 @@ void DigikamApp::setupActions()
                                    SLOT(slotToggleFullScreen()),
                                    actionCollection(),
                                    "full_screen");
-
+    mFullScreenAction->setWhatsThis(i18n("This option allows you to toggle the main windows in full screen mode."));    
+    
     mQuitAction = KStdAction::quit(this,
                                    SLOT(slot_exit()),
                                    actionCollection(),
                                    "app_exit");
 
-    mKipiHelpAction = new KAction(i18n("Kipi Plugins' Handbooks"),
+    mKipiHelpAction = new KAction(i18n("Kipi Plugins Handbook"),
                                    "kipi",
                                    0,
                                    this,
@@ -476,7 +552,6 @@ void DigikamApp::setupActions()
 
     mImageViewAction->setEnabled(false);
     mImageCommentsAction->setEnabled(false);
-    mImageViewExifAction->setEnabled(false);
     mImageRenameAction->setEnabled(false);
     mImageDeleteAction->setEnabled(false);
     mImagePropsAction->setEnabled(false);
@@ -487,7 +562,6 @@ void DigikamApp::setupActions()
     mImageSortAction->setCurrentItem((int)mAlbumSettings->getImageSortOrder());
 }
 
-
 void DigikamApp::enableThumbSizePlusAction(bool val)
 {
     mThumbSizePlusAction->setEnabled(val);
@@ -498,25 +572,86 @@ void DigikamApp::enableThumbSizeMinusAction(bool val)
     mThumbSizeMinusAction->setEnabled(val);
 }
 
+void DigikamApp::enableAlbumBackwardHistory(bool enable)
+{
+    mBackwardActionMenu->setEnabled(enable);
+}
+
+void DigikamApp::enableAlbumForwardHistory(bool enable)
+{
+    mForwardActionMenu->setEnabled(enable);
+}
+
+void DigikamApp::slotAboutToShowBackwardMenu()
+{
+    mBackwardActionMenu->popupMenu()->clear();
+    QStringList titles;
+    mView->getBackwardHistory(titles);
+    if(!titles.isEmpty())
+    {
+        int id = 1;
+        QStringList::Iterator iter = titles.begin();        
+        for(; iter != titles.end(); ++iter,++id)
+        {
+            mBackwardActionMenu->popupMenu()->insertItem(*iter, id);
+        }        
+    }
+}
+
+void DigikamApp::slotAboutToShowForwardMenu()
+{
+    mForwardActionMenu->popupMenu()->clear();
+    QStringList titles;
+    mView->getForwardHistory(titles);
+    if(!titles.isEmpty())
+    {
+        int id = 1;
+        QStringList::Iterator iter = titles.begin();
+        for(; iter != titles.end(); ++iter,++id)
+        {
+            mForwardActionMenu->popupMenu()->insertItem(*iter, id);
+        }        
+    }
+}
+
 void DigikamApp::slot_albumSelected(bool val)
 {
-    mDeleteAction->setEnabled(val);
-    mAddImagesAction->setEnabled(val);
-    mPropsEditAction->setEnabled(val);
-    mOpenInKonquiAction->setEnabled(val);
+    Album *album = mAlbumManager->currentAlbum();
+    if(album && !album->isRoot())
+    {   
+        mDeleteAction->setEnabled(val);
+        mAddImagesAction->setEnabled(val);
+        mPropsEditAction->setEnabled(val);
+        mOpenInKonquiAction->setEnabled(val);
+    }
+    else
+    {
+        mDeleteAction->setEnabled(false);
+        mAddImagesAction->setEnabled(false);
+        mPropsEditAction->setEnabled(false);
+        mOpenInKonquiAction->setEnabled(false);
+    }
 }
 
 void DigikamApp::slot_tagSelected(bool val)
 {
-    mDeleteTagAction->setEnabled(val);
-    mEditTagAction->setEnabled(val);
+    Album *album = mAlbumManager->currentAlbum();
+    if(album && !album->isRoot())
+    {
+        mDeleteTagAction->setEnabled(val);
+        mEditTagAction->setEnabled(val);
+    }
+    else
+    {
+        mDeleteTagAction->setEnabled(false);
+        mEditTagAction->setEnabled(false);
+    }
 }
 
 void DigikamApp::slot_imageSelected(bool val)
 {
     mImageViewAction->setEnabled(val);
     mImageCommentsAction->setEnabled(val);
-    mImageViewExifAction->setEnabled(val);
     mImageRenameAction->setEnabled(val);
     mImageDeleteAction->setEnabled(val);
     mImagePropsAction->setEnabled(val);
@@ -551,6 +686,10 @@ void DigikamApp::slotCameraConnect()
         CameraUI* cgui = new CameraUI(this, ctype->title(), ctype->model(),
                                       ctype->port(), ctype->path());
         cgui->show();
+        connect(cgui, SIGNAL(signalLastDestination(const KURL&)),
+                mView, SLOT(slotSelectAlbum(const KURL&)));
+        connect(cgui, SIGNAL(signalAlbumSettingsChanged()),
+                SLOT(slotSetupChanged()));
     }
 }
 
@@ -576,46 +715,66 @@ void DigikamApp::slotCameraRemoved(CameraType *ctype)
         mCameraMenuAction->remove(cAction);
 }
 
+void DigikamApp::slotCameraAutoDetect()
+{
+    bool retry = false;
+    
+    CameraType* ctype = mCameraList->autoDetect(retry);
+    if (!ctype && retry)
+    {
+        QTimer::singleShot(0, this, SLOT(slotCameraAutoDetect()));
+        return;
+    }
+    
+    if (ctype && ctype->action())
+    {
+        ctype->action()->activate();
+    }
+}
+
 void DigikamApp::slotSetup()
 {
-    m_setup = new Setup;
-
-    connect(m_setup, SIGNAL(okClicked()),
-            this,  SLOT(slotSetupChanged()));
+    Setup setup(this);
 
     // For to show the number of KIPI plugins in the setup dialog.
 
     KIPI::PluginLoader::PluginList list = KipiPluginLoader_->pluginList();
-    m_setup->pluginsPage_->initPlugins((int)list.count());
+    setup.pluginsPage_->initPlugins((int)list.count());
 
-    m_setup->show();
+    if (setup.exec() != QDialog::Accepted)
+        return;
+
+    setup.pluginsPage_->applyPlugins();
+    m_ImagePluginsLoader->loadPluginsFromList(setup.editorPage_->getImagePluginsListEnable());
+
+    slotSetupChanged();
 }
 
 void DigikamApp::slotSetupCamera()
 {
-    m_setup = new Setup(this, 0, Setup::Camera);
-
-    connect(m_setup, SIGNAL(okClicked()),
-            this,  SLOT(slotSetupChanged()));
+    Setup setup(this, 0, Setup::Camera);
 
     // For to show the number of KIPI plugins in the setup dialog.
 
     KIPI::PluginLoader::PluginList list = KipiPluginLoader_->pluginList();
-    m_setup->pluginsPage_->initPlugins((int)list.count());
+    setup.pluginsPage_->initPlugins((int)list.count());
 
-    m_setup->show();
+    if (setup.exec() != QDialog::Accepted)
+        return;
+
+    setup.pluginsPage_->applyPlugins();
+    m_ImagePluginsLoader->loadPluginsFromList(setup.editorPage_->getImagePluginsListEnable());
+
+    slotSetupChanged();
 }
 
 void DigikamApp::slotSetupChanged()
 {
-    m_setup->pluginsPage_->applyPlugins();
-    m_ImagePluginsLoader->loadPluginsFromList(m_setup->editorPage_->getImagePluginsListEnable());
-
     mAlbumManager->setLibraryPath(mAlbumSettings->getAlbumLibraryPath());
     mView->applySettings(mAlbumSettings);
     updateDeleteTrashMenu();
-    if (ImageWindow::instance())
-        ImageWindow::instance()->applySettings();
+    if (ImageWindow::imagewindow())
+        ImageWindow::imagewindow()->applySettings();
     m_config->sync();
 }
 
@@ -710,7 +869,7 @@ void DigikamApp::loadPlugins()
 
     KipiPluginLoader_->loadPlugins();
 
-    m_ImagePluginsLoader = new ImagePluginLoader(this);
+    m_ImagePluginsLoader = new ImagePluginLoader(this, mSplash);
 }
 
 void DigikamApp::slotKipiPluginPlug()
@@ -729,15 +888,24 @@ void DigikamApp::slotKipiPluginPlug()
     m_kipiBatchActions.clear();
     m_kipiAlbumActions.clear();
 
+    m_kipiAlbumActions.append( mAddImagesAction );
+
     KIPI::PluginLoader::PluginList list = KipiPluginLoader_->pluginList();
+    
+    int cpt = 0;
 
     for( KIPI::PluginLoader::PluginList::Iterator it = list.begin() ; it != list.end() ; ++it )
-        {
+    {
         KIPI::Plugin* plugin = (*it)->plugin();
 
         if ( !plugin || !(*it)->shouldLoad() )
             continue;
 
+        ++cpt;
+        
+        if(mSplash)
+            mSplash->message(i18n("Loading: %1").arg((*it)->name()));
+
         plugin->setup( this );
         QPtrList<KAction>* popup = 0;
 
@@ -746,7 +914,7 @@ void DigikamApp::slotKipiPluginPlug()
         KActionPtrList actions = plugin->actions();
 
         for( KActionPtrList::Iterator it2 = actions.begin(); it2 != actions.end(); ++it2 )
-            {
+        {
             if ( plugin->category(*it2) == KIPI::IMAGESPLUGIN )
                popup = &m_kipiImageActions;
 
@@ -771,11 +939,14 @@ void DigikamApp::slotKipiPluginPlug()
                popup->append( *it2 );
             else
                kdDebug() << "No menu found for a plugin!!!" << endl;
-            }
-
-        plugin->actionCollection()->readShortcutSettings();
         }
 
+        plugin->actionCollection()->readShortcutSettings();
+    }
+    
+    if(mSplash)
+        mSplash->message(i18n("1 Kipi Plugin Loaded", "%n Kipi Plugins Loaded", cpt));
+    
     // Create GUI menu in according with plugins.
 
     plugActionList( QString::fromLatin1("file_actions_export"), m_kipiFileActionsExport );
